@@ -292,7 +292,6 @@ private:
 		rtpproxy *rtp;
 
 		void reply_source(int error);
-		void decline(thread *thread);
 		void ring(thread *thread, session *s = NULL);
 		void busy(thread *thread, session *s = NULL);
 		void failed(thread *thread, session *s);
@@ -448,7 +447,7 @@ public:
 	static void activate(MappedRegistry *rr);
 	static void expire(MappedRegistry *rr);
 	static void registration(int id, modules::regmode_t mode);
-	static void authenticate(int id, const char *realm);
+	static bool authenticate(int id, const char *realm);
 	static const char *referLocal(MappedRegistry *rr, const char *target, char *buffer, size_t size);
 	static const char *referRemote(MappedRegistry *rr, const char *target, char *buffer, size_t size);
 
@@ -525,6 +524,10 @@ private:
 	unsigned via_port;
 	rtpproxy::session proxyinfo;
 
+	char *sip_realm;
+	osip_proxy_authenticate_t *proxy_auth;
+	osip_www_authenticate_t *www_auth;
+
 	enum {EXTERNAL, LOCAL, PUBLIC, ROUTED, FORWARDED} destination;
 	enum {CALL, MESSAGE, REGISTRAR, NONE} authorizing;
 
@@ -541,6 +544,7 @@ private:
 	bool getsource(void);
 	bool unauthenticated(void);
 	bool authenticate(void);
+	bool authenticate(stack::session *session);
 	bool authorize(void);
 	void registration(void);
 	void validate(void);
